function [SNR] = Demo1D()
clear
clc
M = [0.1:0.1:0.8];
SignalSize = 256;
NoTrial = 7;
SNR = zeros(NoTrial,3);
try
    matlabpool(7);
end
for i=1:length(M)
    display(['Processing for ' num2str(M(i)*100),' % Measurements'])
    display('-------------------------------------------');
    temp = zeros(NoTrial,3);
    parfor j = 1:NoTrial
        tic
        temp(j,:) = Sim1D(M(i),SignalSize);
        display(['Processed Trial No ', num2str(j),' of ' num2str(NoTrial), ' in ',num2str(toc),' seconds']);
    end
    SNR(i,:) = sum(temp)/NoTrial
    display('+++++++++++++++++++++++++++++++++++++++++++');
end

figure
hold on
plot(M,SNR(:,1),'y');
hold on
plot(M,SNR(:,2),'r');
plot(M,SNR(:,3),'g')

function [SNR] = Sim1D(M,N)
%M is the Sparsity Level Coefficient

%Initialize the problem variables
warning off
close all

thres = 10^-2;
Amp = 1;
NLoop = 2;

%% Generate the signal and its DCT
%The original Signal
S = cusp_gen(N);
x_org = dct(S);
T = dctmtx(N);
Ti = T^-1;
x_org(find(abs(x_org)<thres))=0;
x_org = x_org*Amp;

%Find how sparse the signal is
Sp =length(find(abs(x_org)==0));
Splevel = N-Sp;
x_org = dct(S);
A = randn((N)*M,N)*Ti;
A = orth(A')';
b = (A*x_org);
%b = awgn(b,1);

%% Start the Iterations
x0 = pinv(A)*b;
x0=zeros(size(x0));
t = clock;
xL2 = x0;
xEnt = x0;
p = 1;
for j =1:length(b)*NLoop
    for i = 1:size(A,1)
        xEnt = EntropicProj(A(i,:),b(i),xEnt);
        xL2 = L2Proj(A(i,:),b(i),xL2);
    end
    [~,indxL2] =sort(abs(xL2),'descend');
    [~,indxEnt] =sort(abs(xEnt),'descend');
    if p > length(b)
        p = length(x_org);
    end
    xL2(indxL2(p+1:end)) = 0;
    xEnt(indxEnt(p+1:end)) = 0;
    p = p+1;
end
[xCoSamp]=cosamp(A,b,Splevel,10^-5,1000);
SNR(1) = snr(S,Ti*xCoSamp/Amp);
SNR(2) = snr(S,Ti*(xL2/Amp));
SNR(3) = snr(S,Ti*(xEnt/Amp));

function [S] = cusp_gen(n)
n = (1:n) ./n;
S = sqrt(abs(n - .37))';